//-----------------------------------------------------
// Pic Pack library
// 
// serial_demo.c
//
// Demonstration routines for the interrupt driven
// serial library
//
// Ian Harris 2010
// www.embeddedadventures.com
//
//-----------------------------------------------------


// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"

// System includes
#include "string.h"
#include "stdlib.h"
#include "config.h"
#include "pic_term.h"


// Dummy variable to show we can set it 

uns8 my_var = 0;

// Interrupt routine - - - - - - - - - -

void interrupt() {
	
		serial_handle_tx_isr();
		serial_handle_rx_isr();
}

//
// term_entry_callback will be called when 
// user pressed <enter> on serial "terminal".
// term_buffer contains the characters (if any)
// that the user has entered.

void term_entry_callback(uns8 *term_buffer) {

uns8 data[5];
uns8 first_char;
	
	first_char = term_buffer[0];

	switch (first_char) {
		case 'a':
			serial_print_str("a: print my_var=");
			serial_print_int(my_var);
			serial_print_str("\n");
			break;
		case 'x':
			serial_print_str("x: print my_var=0x");
			serial_print_int_hex(my_var);
			serial_print_str("\n");
			break;
		case 'c':
			serial_print_str("You requested command c\n");
			break;
		case 'n':
			serial_print_str("n is an interesting command\n");
			break;
		case 's':
			// More complicated example
			// Let's set a variable based on input
			// You can type:
			// s05
			// And press enter, it will set "my_var" to this (in hex)
			if (strlen(term_buffer)==3) {
				my_var = atoui_hex(&term_buffer[1]);
				serial_print_str(" Set my_var to ");
				serial_print_int(my_var);	// Of course, this could be anything
				serial_putc('\n');
			} else {
				serial_print_str("Must use sxx where xx is two hex digits\n");
			}
			break;
		default:
			serial_print_str("Unknown command.\n");
			break;
	}	// switch		

}	// term_entry_callback


// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	
	// turn off interrupts just in case
	kill_interrupts();
	
	// kill those pesky analogue inputs
	turn_analog_inputs_off();	
	
	// set serial to run at 9600bps 
	serial_setup(SPBRG_9600);
	
	// initialise termainl
	term_init();

	// tern peripheral interrupts on (ie, serial port)
	turn_peripheral_ints_on();
	
	// turn all interrupts on
	turn_global_ints_on();
}



void main() {
	
	configure_system();
	
	delay_ms(100);	// mostly so screamer bootloader has time to switch windows
	
	serial_print_str("\n\nPIC terminal\n");
	serial_print_str( "\n<");
	
	// Print the time and date this file was compiled. Handy to know which
	// version you are running.
	
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	serial_print_str("Commands: a,x,c,n,s00\n");

	// a,x,c,n just press the key and enter
	// s00 sets the variable "my_var" to the hex value you enter


	for(;;) {

		term_process();
	
	}

	
}	// main

