//-----------------------------------------------------
// Pic Pack library
// 
// simon_game.c
//
// Simon game using PicPack library
//
// Ian Harris 2009
// imharris [at] gmail.com
//
// Released under the "do whatever you like with this
// but if it breaks, you get to keep both pieces.
//-----------------------------------------------------

// Simon game includes
#include "phrases.h"
#include "simon_lights.h"

// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
#include "somo_14d.h"
#include "audio_queue.h"
#include "pic_tick.h"
#include "pic_timer.h"
 

// System includes
#include "string.h"
#include "stdlib.h"
#include "config.h"
#include "rand.h"

// Put received characters in here to see
// if they make a command
char serial_buffer[6]= '\0';

// See if we need to do a task
uns8 do_task = 0;

// Flag if we received something
bit receive = 0;


// Game play defines

// Game rating levels
// you'll get this level if you reach the number
// of repeats

#define LEVEL_5 10
#define LEVEL_4 8
#define LEVEL_3 6
#define LEVEL_2 3
#define LEVEL_1 0


// Game states

enum _enum_states {
	ST_NONE = 0,
	ST_WELCOME,
	ST_CHOOSE_PLAYER_RED,
	ST_CHOOSE_PLAYER_BLUE,
	ST_CHOOSE_PLAYER_GREEN,
	ST_CHOOSE_PLAYER_YELLOW,
	ST_CHOOSE_PLAYER_WAIT,
	ST_CHOSEN_PLAYER,
	ST_SIMON_SAYS_INIT,
	ST_SIMON_SAYS,
	ST_SIMON_SAYS_WAIT,
	ST_PLAYER_REPEAT,
	ST_SIMON_SAYS_PRE_DELAY,
	ST_PLAYER_REPEAT_KEYPRESS,
	ST_PLAYER_REPEAT_LIGHT_OFF,
	ST_PLAYER_REPEAT_TIMEOUT,
	ST_SHOW_CORRECT_ANSWER,
	ST_SHOW_RATING_1,
	ST_SHOW_RATING_2,
	ST_SHOW_RATING_WAIT,
	ST_PLAY_WITH_ME,
	ST_HEY,
	ST_COME_ON,
	
};

typedef enum _enum_states enum_states;

// wait for 

#define WF_AUDIO_QUEUE_FINISH	0x01
#define WF_TIMEOUT				0x02
#define WF_KEY_PRESS			0x04

// keys


#define KEY_RED		0x01
#define KEY_BLUE	0x02
#define KEY_GREEN	0x04
#define	KEY_YELLOW	0x08

#define KEY_WHITE	0x10 

// game play

#define GAME_PRESS_RED		0
#define GAME_PRESS_BLUE		1
#define GAME_PRESS_GREEN	2
#define GAME_PRESS_YELLOW	3

uns8 game[50];
uns8 simon_says_count;
uns8 simon_says_level;


uns8 game_mode;
uns8 player;
enum_states state = ST_NONE;
enum_states next_state_on_audio_queue_finish;
enum_states next_state_on_keypress;
enum_states next_state_on_timeout;

uns8 wait_for;
uns8 event;
uns8 last_key_state = 0;
uns8 choose_player_count = 0;

uns8 time_out = 0;
uns8 count;

uns16 timeout_start;
uns16 timeout_wait;
uns16 timeout_busy;

void randomise_game() {
	
	// need to source a random counter value
	
	srand(tick_get_count() & 0xff);
 
	for (count = 0; count < 50; count++) {
		game[count] = rand() & 0x03;
	}
}	

void set_wait_timeout(uns16 milliseconds) {
	timeout_start = tick_get_count();
	timeout_wait = milliseconds;
}	


uns8 get_key_state() {


uns8 key = 0;

	if (!test_pin(red_button_port, red_button_pin)) {
		key  = key | KEY_RED;
	}
	if (!test_pin(green_button_port, green_button_pin)) {
		key  = key | KEY_GREEN;
	}
	if (test_pin(blue_button_port, blue_button_pin)) {
		key  = key | KEY_BLUE;
	}
	if (!test_pin(yellow_button_port, yellow_button_pin)) {
		key  = key | KEY_YELLOW;
	}
	if (!test_pin(white_button_port, white_button_pin)){
		key  = key | KEY_WHITE;
	}
	return key;

}

uns8 last_key_pressed() {
	uns8 key_press = 0;
	uns8 new_key_state = get_key_state();
	uns8 key = KEY_RED;
	uns8 count = 0;
	while (count < 4) { 
		if ((new_key_state & key) && ((last_key_state & key) == 0)) {
			delay_ms(10);
			new_key_state = get_key_state();
			if (new_key_state & key) {
				key_press = key_press | key;
				last_key_state = last_key_state | key;
			}
		} else 
		// key up
		if (((new_key_state & key)==0) && (last_key_state & key)) {
			delay_ms(10);
			new_key_state = get_key_state(); 
			if ((new_key_state & key) == 0) {
				last_key_state = last_key_state & (~key);
			}	
		}
		key = key << 1;
		count++;
	}
	return key_press;
}		

uns8 key_pressed() {
	uns8 key_pressed = 0;
	uns8 new_key_state = get_key_state();
	uns8 key = KEY_RED;
	uns8 count = 0;
	while (count < 4) { 
		if ((new_key_state & key) && ((last_key_state & key) == 0)) {
			delay_ms(10);
			new_key_state = get_key_state();
			if (new_key_state & key) {
				key_pressed = 1;
			}	
		} else
		if (((new_key_state & key)==0) && (last_key_state & key)) {
			delay_ms(10);
			new_key_state = get_key_state();
			if ((new_key_state & key) == 0) {
				last_key_state = last_key_state & (~key);
			}	
		}
		key = key << 1;
		count++;
	}
	return key_pressed;
}		



void print_state() {
	switch (state) {
		case ST_NONE: serial_print_str("ST_NONE"); break;
		case ST_WELCOME: serial_print_str("ST_WELCOME"); break;
		case ST_CHOOSE_PLAYER_RED: serial_print_str("ST_CHOOSE_PLAYER_RED"); break;
		case ST_CHOOSE_PLAYER_BLUE: serial_print_str("ST_CHOOSE_PLAYER_BLUE"); break;
		case ST_CHOOSE_PLAYER_GREEN: serial_print_str("ST_CHOOSE_PLAYER_GREEN"); break;
		case ST_CHOOSE_PLAYER_YELLOW: serial_print_str("ST_CHOOSE_PLAYER_YELLOW"); break;
		case ST_CHOOSE_PLAYER_WAIT: serial_print_str("ST_CHOOSE_PLAYER_WAIT"); break;
		case ST_CHOSEN_PLAYER: serial_print_str("ST_CHOSEN_PLAYER"); break;
		case ST_SIMON_SAYS_INIT: serial_print_str("ST_SIMON_SAYS_INIT"); break;
		case ST_SIMON_SAYS: serial_print_str("ST_SIMON_SAYS"); break;
		case ST_SIMON_SAYS_WAIT: serial_print_str("ST_SIMON_SAYS_WAIT"); break;
		case ST_PLAYER_REPEAT: serial_print_str("ST_PLAYER_REPEAT"); break;
		case ST_SIMON_SAYS_PRE_DELAY: serial_print_str("ST_SIMON_SAYS_PRE_DELAY"); break;
		case ST_PLAYER_REPEAT_KEYPRESS: serial_print_str("ST_PLAYER_REPEAT_KEYPRESS"); break;
		case ST_PLAYER_REPEAT_LIGHT_OFF: serial_print_str("ST_PLAYER_REPEAT_LIGHT_OFF"); break;
		case ST_PLAYER_REPEAT_TIMEOUT: serial_print_str("ST_PLAYER_REPEAT_TIMEOUT"); break;
		case ST_SHOW_CORRECT_ANSWER: serial_print_str("ST_SHOW_CORRECT_ANSWER"); break;
		case ST_SHOW_RATING_1: serial_print_str("ST_SHOW_RATING_1"); break;
		case ST_SHOW_RATING_2: serial_print_str("ST_SHOW_RATING_2"); break;
		case ST_SHOW_RATING_WAIT: serial_print_str("ST_SHOW_RATING_WAIT"); break;
		case ST_PLAY_WITH_ME: serial_print_str("ST_PLAY_WITH_ME"); break;	
		case ST_HEY: serial_print_str("ST_HEY"); break;	
		case ST_COME_ON: serial_print_str("ST_COME_ON"); break;	


	}

}


void run_state() {

uns8 key = last_key_pressed();

	serial_print_str("\n\nState = ");
	print_state();
	serial_print_nl();
	switch (state) {
		case ST_NONE:
		case ST_WELCOME:
			state = ST_WELCOME;
			simon_lights_set(LT_ALL_ON);
			audio_queue_add(PH_WELCOME);
			choose_player_count = 0;
	
			wait_for = WF_AUDIO_QUEUE_FINISH | WF_KEY_PRESS;
			next_state_on_audio_queue_finish = ST_CHOOSE_PLAYER_RED;
			next_state_on_keypress = ST_CHOSEN_PLAYER;

			break;
		case ST_CHOOSE_PLAYER_RED:
			simon_lights_set(LT_RED_FLASH);
			audio_queue_add(PH_IF_YOUR_NAME);
			audio_queue_add(player_a_phrase);
			audio_queue_add(PH_PRESS_THE);
			audio_queue_add(PH_RED);
			audio_queue_add(PH_BUTTON);
			next_state_on_audio_queue_finish = ST_CHOOSE_PLAYER_GREEN;
			next_state_on_keypress = ST_CHOSEN_PLAYER;
			wait_for = WF_AUDIO_QUEUE_FINISH | WF_KEY_PRESS;
			break;
		case ST_CHOOSE_PLAYER_GREEN:	
			simon_lights_set(LT_GREEN_FLASH);
			audio_queue_add(PH_IF_YOUR_NAME);
			audio_queue_add(player_b_phrase);
			audio_queue_add(PH_PRESS_THE);
			audio_queue_add(PH_GREEN);
			audio_queue_add(PH_BUTTON);
			next_state_on_audio_queue_finish = ST_CHOOSE_PLAYER_BLUE;
			next_state_on_keypress = ST_CHOSEN_PLAYER;
			wait_for = WF_AUDIO_QUEUE_FINISH | WF_KEY_PRESS;
			break;
		case ST_CHOOSE_PLAYER_BLUE:	
			simon_lights_set(LT_BLUE_FLASH);
			audio_queue_add(PH_IF_YOUR_NAME);
			audio_queue_add(player_c_phrase);
			audio_queue_add(PH_PRESS_THE);
			audio_queue_add(PH_BLUE);
			audio_queue_add(PH_BUTTON);
			next_state_on_audio_queue_finish = ST_CHOOSE_PLAYER_YELLOW;
			next_state_on_keypress = ST_CHOSEN_PLAYER;
			wait_for = WF_AUDIO_QUEUE_FINISH | WF_KEY_PRESS;
			break;
		case ST_CHOOSE_PLAYER_YELLOW:
			simon_lights_set(LT_YELLOW_FLASH);
			audio_queue_add(PH_IF_YOUR_NAME);
			audio_queue_add(player_d_phrase);
			audio_queue_add(PH_PRESS_THE);
			audio_queue_add(PH_YELLOW);
			audio_queue_add(PH_BUTTON);
			wait_for = WF_AUDIO_QUEUE_FINISH | WF_KEY_PRESS;
			next_state_on_audio_queue_finish = ST_CHOOSE_PLAYER_WAIT;
			next_state_on_keypress = ST_CHOSEN_PLAYER;
			break;
		case ST_CHOOSE_PLAYER_WAIT:
			choose_player_count++;
			if (choose_player_count > 2) {
				next_state_on_timeout = ST_PLAY_WITH_ME;
			} else {
				next_state_on_timeout = ST_CHOOSE_PLAYER_RED;
			}
			simon_lights_set(LT_ALL_OFF);
			set_wait_timeout(5000);
			wait_for = WF_KEY_PRESS | WF_TIMEOUT;
			next_state_on_keypress = ST_CHOSEN_PLAYER;
			break;
		case ST_CHOSEN_PLAYER:
			audio_queue_clear();
			simon_lights_set(LT_ALL_OFF);
			// received keypress - but which one?
			if (key & KEY_RED) {
				player = 1;
				audio_queue_add(player_a_phrase);
				simon_lights_set(LT_RED);
			} else if (key & KEY_GREEN) {
				player = 2;
				audio_queue_add(player_b_phrase);
				simon_lights_set(LT_GREEN);
			} else if (key & KEY_BLUE) {
				player = 3;
				simon_lights_set(LT_BLUE);
				audio_queue_add(player_c_phrase);
			} else {
				player = 4;
				simon_lights_set(LT_YELLOW);
				audio_queue_add(player_d_phrase);
			}
			audio_queue_add(PH_ARE_YOU_READY);
			next_state_on_audio_queue_finish = ST_SIMON_SAYS_INIT;
			wait_for = WF_AUDIO_QUEUE_FINISH;
			break;
		case ST_PLAY_WITH_ME:
			audio_queue_add(PH_WAIT);
			set_wait_timeout(10000);
			wait_for = WF_TIMEOUT |WF_KEY_PRESS;
			// next_state_on_audio_queue_finish = ST_CHOOSE_PLAYER_RED;
			next_state_on_keypress = ST_CHOSEN_PLAYER;
			next_state_on_timeout  = ST_HEY;
			break;
		case ST_HEY:
			audio_queue_add(PH_HEY);
			set_wait_timeout(15000);
			wait_for = WF_TIMEOUT |WF_KEY_PRESS;
			// next_state_on_audio_queue_finish = ST_CHOOSE_PLAYER_RED;
			next_state_on_keypress = ST_CHOSEN_PLAYER;
			next_state_on_timeout  = ST_COME_ON;
			break;
		case ST_COME_ON:
			audio_queue_add(PH_COME_ON);
			set_wait_timeout(15000);
			wait_for = WF_TIMEOUT |WF_KEY_PRESS;
			// next_state_on_audio_queue_finish = ST_CHOOSE_PLAYER_RED;
			next_state_on_keypress = ST_CHOSEN_PLAYER;
			next_state_on_timeout  = ST_CHOOSE_PLAYER_RED;
			break;
						
		case ST_SIMON_SAYS_INIT:
			simon_says_level = 0;
			simon_says_count = 0;
			// randomise game
			randomise_game();
			state = ST_SIMON_SAYS_PRE_DELAY;	// just for completeness
			// NO BREAK
		case ST_SIMON_SAYS_PRE_DELAY:
			simon_lights_set(LT_ALL_OFF);
			set_wait_timeout(1000);	// 1s
			wait_for = WF_TIMEOUT;
			next_state_on_timeout = ST_SIMON_SAYS;
			break;
		case ST_SIMON_SAYS:

			if (game[simon_says_count] == GAME_PRESS_RED) {
				simon_lights_set(LT_RED);
				audio_queue_add(PH_RED);
			} else if (game[simon_says_count] == GAME_PRESS_BLUE) {
				simon_lights_set(LT_BLUE);
				audio_queue_add(PH_BLUE);
			} else if (game[simon_says_count] == GAME_PRESS_GREEN) {
				simon_lights_set(LT_GREEN);
				audio_queue_add(PH_GREEN);
			} else {	// must be yellow!
				simon_lights_set(LT_YELLOW);
				audio_queue_add(PH_YELLOW);
			}
			if (simon_says_count < simon_says_level) {
				simon_says_count++;
				wait_for = WF_AUDIO_QUEUE_FINISH;
				next_state_on_audio_queue_finish = ST_SIMON_SAYS_WAIT;	// Do it again!
			} else {	// We have done all our simon says... so their turn!
				simon_says_count = 0;
				wait_for = WF_AUDIO_QUEUE_FINISH;
				next_state_on_audio_queue_finish = ST_PLAYER_REPEAT;
			}
			break;
		case ST_SIMON_SAYS_WAIT:
			simon_lights_set(LT_ALL_OFF);
			if (simon_says_level < 4) {
				set_wait_timeout(200);
			} else if (simon_says_level < 8) {
				set_wait_timeout(150);
			} else if (simon_says_level < 12) {
				set_wait_timeout(100);
			} else {
				set_wait_timeout(50);
			}
			wait_for = WF_TIMEOUT;
			next_state_on_timeout = ST_SIMON_SAYS;
			break;
		case ST_PLAYER_REPEAT:
			simon_lights_set(LT_ALL_OFF);
			set_wait_timeout(5000); 	// wait five seconds for answer
			wait_for = WF_TIMEOUT | WF_KEY_PRESS;
			next_state_on_keypress = ST_PLAYER_REPEAT_KEYPRESS;
			next_state_on_timeout  = ST_PLAYER_REPEAT_TIMEOUT;
			break;
		case ST_PLAYER_REPEAT_KEYPRESS:

			
			// Got a keypress, but is it the right one?
			uns8 expected_answer = game[simon_says_count];
			
			if ((key == KEY_RED) && (expected_answer == GAME_PRESS_RED)) {
				audio_queue_clear();
				audio_queue_add(PH_RED);
				simon_lights_set(LT_RED);
			} else if ((key == KEY_BLUE) && (expected_answer == GAME_PRESS_BLUE)) {
				audio_queue_clear();
				audio_queue_add(PH_BLUE);
				simon_lights_set(LT_BLUE);
			} else if ((key == KEY_GREEN) && (expected_answer == GAME_PRESS_GREEN)) {
				audio_queue_clear();
				audio_queue_add(PH_GREEN);
				simon_lights_set(LT_GREEN);
			} else if ((key == KEY_YELLOW) && (expected_answer == GAME_PRESS_YELLOW)) {
				audio_queue_clear();
				audio_queue_add(PH_YELLOW);
				simon_lights_set(LT_YELLOW);
			} else {	// got it wrong!
				audio_queue_add(PH_WRONG);
				switch (key) {
					case KEY_RED:	simon_lights_set(LT_RED); break;
					case KEY_BLUE:	simon_lights_set(LT_BLUE); break;
					case KEY_GREEN: simon_lights_set(LT_GREEN); break;
					case KEY_YELLOW: simon_lights_set(LT_YELLOW); break;
				}
				wait_for = WF_AUDIO_QUEUE_FINISH;
				next_state_on_audio_queue_finish = ST_SHOW_CORRECT_ANSWER;
				break; // nothing else to do
			}
			// If we're here, then they got it right
			serial_print_str("RIGHT!\n");
			simon_says_count++;
			if (simon_says_count > simon_says_level) {	// finished all
				serial_print_str("All done\n\n");
				simon_says_level++;
				simon_says_count = 0;
				wait_for = WF_AUDIO_QUEUE_FINISH;
				next_state_on_audio_queue_finish = ST_SIMON_SAYS_PRE_DELAY;
			} else {	// not finished all
				serial_print_str("Wait for next key...\n");
				set_wait_timeout(5000); 	// wait five seconds for answer
				wait_for = WF_TIMEOUT | WF_KEY_PRESS | WF_AUDIO_QUEUE_FINISH;
				next_state_on_keypress = ST_PLAYER_REPEAT_KEYPRESS;
				next_state_on_timeout  = ST_PLAYER_REPEAT_TIMEOUT;
				next_state_on_audio_queue_finish = ST_PLAYER_REPEAT_LIGHT_OFF;
			}
			break;
		case ST_PLAYER_REPEAT_LIGHT_OFF:	// or key up event?
			simon_lights_set(LT_ALL_OFF);
			wait_for = WF_TIMEOUT | WF_KEY_PRESS;
			next_state_on_keypress = ST_PLAYER_REPEAT_KEYPRESS;
			next_state_on_timeout  = ST_PLAYER_REPEAT_TIMEOUT;
			break;
		case ST_PLAYER_REPEAT_TIMEOUT:
			simon_lights_set(LT_ALL_OFF);
			audio_queue_add(PH_TOO_LATE);
			wait_for = WF_AUDIO_QUEUE_FINISH;
			next_state_on_audio_queue_finish = ST_SHOW_CORRECT_ANSWER;
			break;
		case ST_SHOW_CORRECT_ANSWER:
			audio_queue_add(PH_CORRECT_ANS);
			switch (game[simon_says_count]) {
				case GAME_PRESS_RED:	simon_lights_set(LT_RED);	audio_queue_add(PH_RED);	break;
				case GAME_PRESS_BLUE:	simon_lights_set(LT_BLUE);	audio_queue_add(PH_BLUE);	break;
				case GAME_PRESS_GREEN:	simon_lights_set(LT_GREEN);	audio_queue_add(PH_GREEN);	break;
				case GAME_PRESS_YELLOW:	simon_lights_set(LT_YELLOW);audio_queue_add(PH_YELLOW);	break;
			}
			wait_for = WF_AUDIO_QUEUE_FINISH;
			next_state_on_audio_queue_finish = ST_SHOW_RATING_1;
			break;
		case ST_SHOW_RATING_1:
			simon_lights_set(LT_ALL_OFF);
			switch (player) {
				case 1: audio_queue_add(PH_PLAYER_1); break;
				case 2: audio_queue_add(PH_PLAYER_2); break;
				case 3: audio_queue_add(PH_PLAYER_5); break;
				case 4: audio_queue_add(PH_PLAYER_6); break;
			}	
			audio_queue_add(PH_YOUR_RATING);
			wait_for = WF_AUDIO_QUEUE_FINISH;
			next_state_on_audio_queue_finish = ST_SHOW_RATING_2;
			break;
		case ST_SHOW_RATING_2:
			if (simon_says_level >= LEVEL_5) {
				audio_queue_add(PH_RATING_5);
				simon_lights_set(LT_ALL_FLASH);
			} else if (simon_says_level >= LEVEL_4) {
				audio_queue_add(PH_RATING_4);
				simon_lights_set(LT_YELLOW_FLASH);
			} else if (simon_says_level >= LEVEL_3) {
				audio_queue_add(PH_RATING_3);
				simon_lights_set(LT_GREEN_FLASH);
			} else if (simon_says_level >= LEVEL_2) {
				audio_queue_add(PH_RATING_2);
				simon_lights_set(LT_BLUE_FLASH);
			} else if (simon_says_level >= LEVEL_1) {
				audio_queue_add(PH_RATING_1);
				simon_lights_set(LT_RED_FLASH);
			}
			wait_for = WF_AUDIO_QUEUE_FINISH;
			next_state_on_audio_queue_finish = ST_SHOW_RATING_WAIT;
			break;
		case ST_SHOW_RATING_WAIT:
			set_wait_timeout(3000);
			wait_for = WF_TIMEOUT;
			next_state_on_timeout = ST_WELCOME;
			break;

		default:
			
			serial_print_str("Strange state unhandled = ");
			serial_print_int(state);
			serial_print_nl();
	}
	
}
		
		



// Interrupt routine - - - - - - - - - -



void interrupt() {
		
		if (intcon.INT0IF) {	// busy signal goes low (no longer busy)
			clear_bit( intcon, INT0IF );
			// see if there is something to process
			delay_ms(10);
			audio_queue_process();
		}
		timer_handle_0_isr();
		serial_handle_tx_isr();
		serial_handle_rx_isr();
	
		

}

// Process_rx
//
// Keep adding characters to our input buffer
// when we enter a command, then act on it

void process_rx() {
	
	uns8 len,	// length of string
		 rec;	// received character
	uns8 first_char;
	
	rec = serial_getc();	// get the character from the fifo
	
	serial_putc(rec);	// print it out so we can see what we typed	// neat trick to allow one button downloads
	
	if (rec == MAGIC_BOOSTBLOADER_REQUEST) {
		boostbloader();
	}

	if (rec == '\r') {	// did we press return?
		first_char = serial_buffer[0];
		switch (first_char) {
			case 'a':	// a = audio finish
				do_task = 1;
				break;
		    case 'r':
				do_task = 2;
				break;
			case 'g':
				do_task = 3;
				break;
			case 'b':
				do_task = 4;
				break;
			case 'y':
				do_task = 5;
				break;
			case 'w':
				do_task = 6;
				break;
			default:
				// More complicated example
				// Let's set a variable based on input
				// You can type:
				// s05
				// And press enter, it will set "send_to" to this (in hex)
				/*if ((first_char == 's') && (strlen(serial_buffer)==3)) {
					send_to = atoui_hex(&serial_buffer[1]);
					serial_print_str(" Set send_to to ");
					serial_print_int(send_to);	// Of course, this could be anything
					serial_putc('\n');
				};*/
		}		
		serial_buffer[0] = '\0';	// clear the buffer
        serial_putc('\n');	// print new line
        serial_putc('>');	// print the prompt
	} else {
		len = strlen(serial_buffer);	// add the character to our line input
		serial_buffer[len] = rec;
		serial_buffer[len+1] = '\0';
	}

}

void handle_tasks() {	

	switch (do_task) {
		case 1:	// s
			//audio_queue_process();
			//audio_queue_done = 1;
			break;
		case 2:	//r
			serial_print_str("<R>");
			//key_pressed = key_pressed | KEY_RED;
			break;
		case 3:	//g
			serial_print_str("<G>");
			//key_pressed = key_pressed | KEY_GREEN;
			break;
		case 4:	//b
			serial_print_str("<B>");
			//key_pressed = key_pressed | KEY_BLUE;
			break;
		case 5:	//y
			serial_print_str("<Y>");
			//key_pressed = key_pressed | KEY_YELLOW;
			break;
		case 6:	//w
			serial_print_str("<W>");
			//key_pressed = key_pressed | KEY_WHITE;
			break;
		
			
	} // switch		

	do_task = 0;
}

// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	    
	kill_interrupts();	// turn off interrupts just in case

	turn_analog_inputs_off();	// kill those pesky analogue inputs
	
	serial_setup(SPBRG_19200);

	#ifdef _PIC16
		timer_setup_0(TIMER_8BIT_MODE, TIMER_PRESCALER_1_TO_8, 0xff - 250 - 1);	// 1ms at 8Mhz
	#endif

	#ifdef _PIC18
		timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 10000);	// 1ms at 40Mhz
		//timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_1_TO_2, 0xffff - 1500 + 7);	// 1ms at 12Mhz
		//timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_1_TO_8, 0xffff - 375 + 1);	// 1ms at 12Mhz
	#endif	
	 
	// int0 connected to busy signal
	set_bit(intcon,INT0IE);	
	// we need to interrupt on low going
	clear_bit(intcon2, INTEDG0);
	
	simon_lights_setup_io();
	
	// Buttons assume weak pull-up
	intcon2.RBPU = 0;
	make_input(red_button_port, red_button_pin);
	make_input(blue_button_port, blue_button_pin);
	make_input(green_button_port, green_button_pin);
	make_input(yellow_button_port, yellow_button_pin);
	make_input(white_button_port, white_button_pin);
	
	somo_14d_setup_io();
	//somo_14d_set_volume(4);
	turn_peripheral_ints_on();
	turn_global_ints_on();

}



void run_game() {

uns16 test_tick;
uns16 tick_marker;

	tick_marker = tick_get_count();
	
	for (;;) {
		run_state();
		
		serial_print_str("Waiting for=");
		serial_print_int(wait_for);
		serial_print_nl();
		while (wait_for !=0 ) {
			if (serial_rx_avail()) {
				process_rx();
			}
			handle_tasks();
			
			test_tick = tick_get_count();	// find out what we're up to

			if (tick_calc_diff(tick_marker, test_tick) >= 250) {	// 250 = 1/4 second
				simon_lights_flash();
				tick_marker = test_tick;
			}
			if ((wait_for & WF_AUDIO_QUEUE_FINISH) && (audio_queue_empty())) {
				serial_print_str(" <AQ> ");
				state = next_state_on_audio_queue_finish;
				wait_for = 0;
			} else if ((wait_for & WF_KEY_PRESS) && (key_pressed())) {
				serial_print_str(" <KP> ");
				state = next_state_on_keypress;
				wait_for = 0;
			} else if (wait_for & WF_TIMEOUT) {
				if (tick_calc_diff(timeout_start, test_tick) >= timeout_wait) {	// has timeout occured?
					serial_print_str(" <TM> ");
					state = next_state_on_timeout;
					wait_for = 0;
				}	
			}
		}	// while
	} // for	
}	


void main() {
	
	configure_system();

	timer_start_0();
	delay_ms(200);
	delay_ms(200);
	delay_ms(200);
	
	serial_print_str("\n\nSimon Game\n");
	serial_print_str( "\n Last change to simon_game.c: <");
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");

	serial_print_str("Commands: r,g,b,y,w <enter>\n");

	run_game();
	
}	// main

