/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/


#include "hmc6352.h"
#include "i2c.h"

void hmc6352_write_eeprom(uns8 addr, uns8 data) {

	// Send hmc6352 addr (write)
	// Send command w (write to eeprom)
	// Send eeprom addr
	// Send data
	
	i2c_start();
	i2c_send_byte(hmc6352_device_addr | hmc6352_write);
	i2c_send_byte(hmc6352_write_to_eeprom);
	i2c_send_byte(addr);
	i2c_send_byte(data);
	i2c_stop();
}
	
uns8 hmc6352_read_eeprom(uns8 addr) {
	
	// Send hmc6352 addr (write)
	// Send command r (read from eeprom)
	// Send eeprom addr
	
	// Send hmc6352 addr (read)
	// Read data

uns8 data;
	
	i2c_start();
	i2c_send_byte(hmc6352_device_addr | hmc6352_write);
	i2c_send_byte(hmc6352_read_from_eeprom);
	i2c_send_byte(addr);
	i2c_stop();

	i2c_start();
	i2c_send_byte(hmc6352_device_addr | hmc6352_read);
	data = i2c_receive_byte();
	i2c_send_ack();
	i2c_stop();

	return data;
}	
	
	
void hmc6352_write_ram(uns8 addr, uns8 data) {

	// Send hmc6352 addr (write)
	// Send command G (write to ram)
	// Send ram addr
	// Send data
	
	i2c_start();
	i2c_send_byte(hmc6352_device_addr | hmc6352_write);
	i2c_send_byte(hmc6352_write_to_ram);
	i2c_send_byte(addr);
	i2c_send_byte(data);
	i2c_stop();

}

	
uns8 hmc6352_read_ram(uns8 addr) {
	
	// Send hmc6352 addr (write)
	// Send command g (read from ram)
	// Send ram addr
	
	// Send hmc6352 addr (read)
	// Read data

uns8 data;
	
	i2c_start();
	i2c_send_byte(hmc6352_device_addr | hmc6352_write);
	i2c_send_byte(hmc6352_read_from_ram);
	i2c_send_byte(addr);
	i2c_stop();

	i2c_start();
	i2c_send_byte(hmc6352_device_addr | hmc6352_read);
	data = i2c_receive_byte();
	i2c_send_ack();
	i2c_stop();

	return data;
}	

void hmc6352_sleep() {

	// Send hmc6352 addr (write)
	// Send command S (sleep)

	i2c_start();
	i2c_send_byte(hmc6352_device_addr | hmc6352_write);
	i2c_send_byte(hmc6352_sleep_cmd);
	i2c_stop();
}	
	
	
void hmc6352_wake() {

	// Send hmc6352 addr (write)
	// Send command W (wake)

	i2c_start();
	i2c_send_byte(hmc6352_device_addr | hmc6352_write);
	i2c_send_byte(hmc6352_wake_cmd);
	i2c_stop();

}	//37208656

void hmc6352_update_bridge_offsets() {
	
	// Send hmc6352 addr (write)
	// Send command O (update bridge offsets)

	i2c_start();
	i2c_send_byte(hmc6352_device_addr | hmc6352_write);
	i2c_send_byte(hmc6352_update_bridge_cmd);
	i2c_stop();
}
	
void hmc6352_enter_cal() {
	
	// Send hmc6352 addr (write)
	// Send command C (Enter user calibration mode)

	i2c_start();
	i2c_send_byte(hmc6352_device_addr | hmc6352_write);
	i2c_send_byte(hmc6352_enter_cal_cmd);
	i2c_stop();	
}

void hmc6352_exit_cal() {

	// Send hmc6352 addr (write)
	// Send command E (Exit user calibration mode)

	i2c_start();
	i2c_send_byte(hmc6352_device_addr | hmc6352_write);
	i2c_send_byte(hmc6352_exit_cal_cmd);
	i2c_stop();
}

void hmc6352_save_op_mode() {
	// Send hmc6352 addr (write)
	// Send command L (Save op mode to eeprom)

	i2c_start();
	i2c_send_byte(hmc6352_device_addr | hmc6352_write);
	i2c_send_byte(hmc6352_save_op_mode_cmd);
	i2c_stop();

}	

uns16 hmc6352_get_data() {
	
	// Send hmc6352 addr (write)
	// Send command A (get data)
	
	// Send hmc6352 addr (read)
	// Read data
	// Read data

uns16 data;
	
	i2c_start();
	i2c_send_byte(hmc6352_device_addr | hmc6352_write);
	i2c_send_byte(hmc6352_get_data_cmd);
	i2c_stop();
	
	i2c_start();
	i2c_send_byte(hmc6352_device_addr | hmc6352_read);
	data = i2c_receive_byte();
	delay_ms(1);
	i2c_send_ack();
	data = data << 8;
	data = data + i2c_receive_byte();
	delay_ms(1);
	i2c_send_ack();
	i2c_stop();

	return data;
}

	

void hmc6352_set_mode(uns8 mode) {
	
uns8 data;
	data = hmc6352_read_ram(0x74);
	
	switch (mode) {
		case hmc6352_mode_standby:
			data.1 = 0;
			data.0 = 0;
			break;
		case hmc6352_mode_query:
			data.1 = 0;
			data.0 = 1;
			break;
		case hmc6352_mode_continuous:
			data.1 = 1;
			data.0 = 0;
			break;
	}
	hmc6352_write_ram(0x74, data);
}

void hmc6352_setup_io() {
	i2c_setup_io();
}	