/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/


#include "spi_hw.h"
#include "pic_serial.h"

void spi_hw_setup_io() {

make_output(PORTC, 5);
make_input(PORTC, 4);

#ifdef SPI_HW_MASTER_MODE
	make_output(PORTC, 3);
	clear_bit(sspcon1, 3);
	clear_bit(sspcon1, 2);
	#ifdef SPI_HW_MASTER_CLOCK_TMR2_DIV_2
		set_bit(sspcon1, 1);
		set_bit(sspcon1, 0);
	#endif
	#ifdef SPI_HW_MASTER_CLOCK_FOSC_DIV_64
		set_bit  (sspcon1, 1);
		clear_bit(sspcon1, 0);
	#endif
	#ifdef SPI_HW_MASTER_CLOCK_FOSC_DIV_16
		clear_bit(sspcon1, 1);
		set_bit  (sspcon1, 0);
	#endif
	#ifdef SPI_HW_MASTER_CLOCK_FOSC_DIV_4
		clear_bit(sspcon1, 1);
		clear_bit(sspcon1, 0);
	#endif
#else
	// Slave mode
	make_input(PORTC, 3);	// sck
	clear_bit(sspcon1, 3);
	set_bit  (sspcon1, 2);
	clear_bit(sspcon1, 1);
	#ifdef SPI_HW_USE_SS
		make_input(PORTA, 5);	// SS
		clear_bit(sspcon1, 0);
	#else
		set_bit(sspcon1, 0);
	#endif
#endif

}	

   
void spi_hw_init() {
	// for 0,0 mode
	clear_bit(sspcon1, CKP);

	set_bit(sspstat, CKE);
	set_bit(sspstat, SMP);	// from tim
	set_bit(sspcon1, SSPEN);	// enable mssp serial port
}

void spi_hw_transmit(uns8 data) {

	clear_bit(pir1, SSPIF);
	// Try and send, if we have something already sending, try again
	do {
		clear_bit(sspcon1, WCOL);
		sspbuf = data;
	} while (test_bit(sspcon1, WCOL));
	
	// Wait here while we transmit
	// Note that we will hang here if we don't have SPI setup properly
	// or we're running under the simulator
	while (!test_bit(pir1, SSPIF)) {}
}

uns8 spi_hw_receive() {

	spi_hw_transmit(0x00);	// dummy transmit to get something back
	return sspbuf;
	
}
	
