/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/


#include "its_common.h"
#include "mrf24j40.h"
#include "pic_serial.h"
#include "wpan.h"

#include "debug.h"

its_device_info its_devices[ITS_MAX_KNOWN_DEVICES];

uns16 its_network_id;
uns16 its_device_id;

uns8 its_sequence = 0;

void its_print_devices() {
	debug_nl();
	for (uns8 count=0; count < ITS_MAX_KNOWN_DEVICES; count++) {
		if (its_devices[count].its_device_id != 0xffff) {	 // 0xffff = blank entry
			debug_str(" dev= ");
			debug_int_hex_16bit(its_devices[count].its_device_id);
			if (its_devices[count].addr.remote.remote_indicator != 0xffff) {
				debug_str(" pan= ");
				debug_int_hex_16bit(its_devices[count].addr.local.pan_id);
				debug_str(" sa= ");
				debug_int_hex_16bit(its_devices[count].addr.local.short_address);
			} else {
				debug_str(" via= ");
				debug_int_hex_16bit(its_devices[count].addr.remote.prior_device_id);
			}
			debug_nl();

		}
	}
}

uns8 its_get_next_sequence() {
	return its_sequence++;
}	

void its_set_network_id(uns16 network_id) {
	its_network_id = network_id;
}


uns16 its_get_network_id() {
	return its_network_id;
}
	
void its_set_device_id(uns16 device_id) {
	its_device_id = device_id;
}
	
uns16 its_get_device_id() {
	return its_device_id;
}
	
void its_init() {
	for (uns8 count=0; count < ITS_MAX_KNOWN_DEVICES; count++) {
		its_devices[count].its_device_id = 0xffff;	 // 0xffff = blank entry
	}
	wpan_init();
}

its_device_info *its_get_device_info(uns8 handle) {
	if (handle < ITS_MAX_KNOWN_DEVICES) {
		return &its_devices[handle];
	} else {
		return NULL;
	}	
}	

its_device_handle its_get_device_handle(uns16 device_id) {

bit found = 0;
	if (device_id == 0xffff) {
		return ITS_DEVICE_NONE;
	}	
	for (uns8 count=0; count < ITS_MAX_KNOWN_DEVICES; count++) {
		if (its_devices[count].its_device_id == device_id) {
			found = 1;
			break;
		}	
	}
	if (found) {
		return count;
	} else {
		return ITS_DEVICE_NONE;
	}
}		

its_device_handle its_add_local_device(uns16 device_id, uns16 pan_id, uns16 short_address) {

bit found_slot = 0;
	debug_str("Adding device_id (");
	debug_int_hex_16bit(device_id);
	debug_str(" (pan_id ");
	debug_int_hex_16bit(pan_id);
	debug_str(" short_addr ");
	debug_int_hex_16bit(short_address);
	debug_str(")\n");
	
	for (uns8 count=0; count < ITS_MAX_KNOWN_DEVICES; count++) {
		if (its_devices[count].its_device_id == 0xffff) {
			found_slot = 1;
			break;
		}	
	}
	if (found_slot) {
		its_devices[count].its_device_id = device_id;
		its_devices[count].addr.local.pan_id = pan_id;
		its_devices[count].addr.local.short_address = short_address;
	} else {
		count = ITS_DEVICE_NONE;
	}
	
	its_print_devices();
	
	return count;

}	

its_device_handle its_add_net_device(uns16 device_id, uns16 previous_hop) {

bit found_slot = 0;
	debug_str(" Add ");
	debug_int_hex_16bit(device_id);
	debug_str(" (via  ");
	debug_int_hex_16bit(previous_hop);
	debug_str(")\n");
	
	for (uns8 count=0; count < ITS_MAX_KNOWN_DEVICES; count++) {
		if (its_devices[count].its_device_id == 0xffff) {
			found_slot = 1;
			break;
		}	
	}
	if (found_slot) {
		its_devices[count].its_device_id = device_id;
		its_devices[count].addr.remote.remote_indicator = 0xffff;
		its_devices[count].addr.remote.prior_device_id = previous_hop;
	} else {
		count = ITS_DEVICE_NONE;
	}
	its_print_devices();
	return count;
}	


void its_transmit_to_handle(its_device_handle handle, uns8 packet_type, uns8 *data, uns8 data_length) {

	// !! should do some checks on handle to see if its valid, but hey...

	its_transmit_to_sa(its_devices[handle].addr.local.pan_id, 
	                   its_devices[handle].addr.local.short_address,
	                   its_devices[handle].its_device_id, 
	                   packet_type, data, data_length);
}

void its_transmit_to_sa(uns16 dest_pan_id, uns16 dest_sa, uns16 dest_device_id, uns8 packet_type, uns8 *data, uns8 data_length) {

uns8 buffer[20];
uns16 temp;
uns8 count;

	buffer[0] = 'I';
	buffer[1] = 'T';
	buffer[2] = 11; 	// counted by hand from below
	buffer[3] = packet_type;
	buffer[4] = its_sequence++;
	// network
	buffer[5] = its_network_id & 0xff;	// lsb
	buffer[6] = its_network_id >> 8;	// msb
	// source
	buffer[7] = its_device_id & 0xff;	// lsb
	buffer[8] = its_device_id >> 8;	// msb
	// dest
	temp = dest_device_id;
	buffer[9] = temp & 0xff;	// lsb
	buffer[10] = temp >> 8;	// msb
	// routing
	// in mode 2 we would go and look up our routing table here
	buffer[11] = 1;	// max hop count
	buffer[12] = 0;	// hop count
	buffer[13] = 0; // num routes is zero
					// so no routes list
	buffer[14] = data_length;	// length of data
	for (count = 0; count < data_length; count++) {
		buffer[15+count] = data[count];
	}	// copy data in
	mrf24j40_transmit_to_short_address(FRAME_TYPE_DATA, dest_pan_id,
	                                   dest_sa, &buffer,
	                                   data_length + 15, MRF_ACK);
}


void its_transmit_to_ea(uns8 *dest_ea, uns16 dest_its_device_id, uns8 packet_type, uns8 *data, uns8 data_length) {

// must do this more efficiently when we have it working!!!!

uns8 buffer[20];

uns8 count;

	// should do some checks on handle to see if its valid, but hey...

	buffer[0] = 'I';
	buffer[1] = 'T';
	buffer[2] = 11; 	// counted by hand from below
	buffer[3] = packet_type;
	buffer[4] = its_sequence++;
	// network
	buffer[5] = its_network_id & 0xff;	// lsb
	buffer[6] = its_network_id >> 8;	// msb
	// source
	buffer[7] = its_device_id & 0xff;	// lsb
	buffer[8] = its_device_id >> 8;	// msb
	// dest
	
	buffer[9] = dest_its_device_id & 0xff;	// lsb
	buffer[10] = dest_its_device_id >> 8;	// msb
	// routing
	// in mode 2 we would go and look up our routing table here
	buffer[11] = 1;	// max hop count
	buffer[12] = 0;	// hop count
	buffer[13] = 0; // num routes is zero
					// so no routes list
	buffer[14] = data_length;	// length of data
	for (count = 0; count < data_length; count++) {
		buffer[15+count] = data[count];
	}	// copy data in
	mrf24j40_transmit_to_extended_address(FRAME_TYPE_DATA, /* pan */ 0xffff,
	                                   dest_ea,  &buffer,
	                                   data_length + 15, MRF_NO_ACK);
}
 