/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/*! \file ar1000.h
    \brief ITS networking routines - common to mode 1 and 2
	
*/
#ifndef __its_common_h
#define __its_common_h

/*

// For your config.h

// - - - - - - - - - - - - - - - - - - - - 
// its common defines
// - - - - - - - - - - - - - - - - - - - - 

#define ITS_MAX_KNOWN_DEVICES 5

*/

#include "pic_utils.h"

// ITS Packet types

#define ITS_ASSOC_REQ			0x00
#define ITS_ASSOC_RES 			0x01
#define ITS_APP_DATA			0x02
#define ITS_ACK					0x03
#define ITS_PENDING_DATA_REQ	0x04
#define ITS_LOCAL_DISCOVER_REQ	0x05
#define ITS_LOCAL_DISCOVER_RES	0x06
#define ITS_NET_DISCOVER_REQ	0x07
#define ITS_NET_DISCOVER_RES	0x08
#define ITS_GENERIC_DATA		0x09
#define ITS_ENDPOINT_REQ		0x0a
// open port
// close port
#define ITS_ENDPOINT_RES		0x0b
// success, failure
#define ITS_ENDPOINT_DATA		0x0c

// Passes sequence ID back, like ack	
#define ITS_ROUTE_FAILURE		0x0d


/** Local address definition */
typedef struct _local_address {
	uns16 pan_id;
	uns16 short_address;
} local_address;

/** Remote address definition */
typedef struct _remote_address {
	uns16 remote_indicator;
	uns16 prior_device_id;
} remote_address;

/** Union of local and remote addresses */
typedef union _its_address {
	local_address      local;
	remote_address     remote;
} its_address;

/** Device info - its_device_id and address (local or remote) */
typedef struct {
	uns16	its_device_id;
	its_address addr;
} its_device_info;

/** Definition of its_device_handle */
typedef uns8 its_device_handle;

#define ITS_DEVICE_NONE 0xff
/**
	\brief Initialise ITS and lower layers
	
	Call before using any ITS functionality
*/	
void its_init();

/**
	\brief Print devices currently known to this one
*/
void its_print_devices();

/**
	\brief Set the ITS network ID
	
	Set the ITS network ID before sending packets
	See also: its_set_device_id(uns16 device_id);

*/

void its_set_network_id(uns16 network_id);

/**
	\brief Retrieve the current network ID
	
*/

uns16 its_get_network_id();

/** 
	\brief Set the ITS device ID
	
	Set the ITS device ID before sending packets. 
	See also: its_set_network_id(uns16 network_id);
*/
void its_set_device_id(uns16 device_id);
uns16 its_get_device_id();
its_device_handle its_get_device_handle(uns16 device_id);
its_device_info   *its_get_device_info(uns8 handle);
its_device_handle its_add_local_device(uns16 device_id, uns16 pan_id, uns16 short_address);
its_device_handle its_add_net_device(uns16 device_id, uns16 previous_hop);
void its_transmit_to_handle(its_device_handle handle, uns8 packet_type, uns8 *data, uns8 data_length);
void its_transmit_to_ea(uns8 *dest_ea,  uns16 dest_its_device_id, uns8 packet_type, uns8 *data, uns8 data_length);
void its_transmit_to_sa(uns16 dest_pan_id, uns16 dest_sa, uns16 dest_device_id, uns8 packet_type, uns8 *data, uns8 data_length);
uns8 its_get_next_sequence();



#endif