/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/



#include "pic_term.h"

uns8 term_buffer[TERM_BUFFER_SIZE];
uns8 buffer_len;

void term_process() {
	
	uns8 len,	// length of string
		 rec;	// received character
	
	if (serial_rx_avail()) {
	
		rec = serial_getc();	// get the character from the fifo
	
		#ifdef TERM_ALLOW_BOOSTBLOADER
			if (rec == MAGIC_BOOSTBLOADER_REQUEST) {
				boostbloader();
			}
		#endif
		
		
		if (rec == '\r') {	// did we press return?
			
			if (buffer_len > 0) {
				term_entry_callback(term_buffer);
			}
			term_buffer[0] = '\0';	// clear the buffer
			buffer_len = 0;
			#ifdef TERM_ECHO_INPUT
				serial_putc('\n');	// print new line
				serial_putc('>');	// print the prompt
			#endif	
		} else {
			if (buffer_len < TERM_BUFFER_SIZE + 2) {	// space for null and new character
				#ifdef TERM_ECHO_INPUT
					serial_putc(rec);	// print it out so we can see what we typed
				#endif
				term_buffer[buffer_len++] = rec;
				term_buffer[buffer_len] = '\0';
			}
		}
	}	
}

void term_init() {
	
	term_buffer[0] = '\0';
	buffer_len = 0;

}
	
