/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/*
 
 \file pic_tick.h
 
 Timer helper routines

	
*/

#ifndef __pic_tick_h
#define __pic_tick_h

#include "pic_utils.h"

/** Global tick counter */
static uns16 tick = 0;

/**
	\brief Return current tick count
	
	Returns the current tick count. Thread and interrupt safe.
*/
uns16 tick_get_count();
/**
	\brief Calculate the tick time difference between two values
	
	Calculates how many ticks have elapsed between two tick values. Covers
	cases where the tick count wraps beyond its 16 bit value.
*/
uns16 tick_calc_diff(uns16 a, uns16 b);

/**
	\brief Call this routine to increment tick count
	
	Typically called during the interrupt routine of a timer to increment
	the tick count.  Note this routine assumes that interupts are off - which
	is always the case in an interrupt sub routine.
*/
void handle_tick();

/**
	\brief Call this routine to increment tick count - inline version
	
	Typically called during the interrupt routine of a timer to increment
	the tick count.  Note this routine assumes that interupts are off - which
	is always the case in an interrupt sub routine. Inline version so you
	don't use up one stack level
*/
inline void handle_tick_inline() {
   tick++;	// we assume that interrupts are off at this point
}   		

#endif   