/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/

/*! \file somo_14d.h
    \brief Somo-14D audio player interface

	Library for accessing the functionality of the 4D systems .ad4 audio player

Put the following into your config.h

// - - - - - - - - - - - - - - - - - - - - 
// somo_14d defines
// - - - - - - - - - - - - - - - - - - - - 

#define somo_14d_clk_port PORTA
#define somo_14d_clk_pin 1

#define somo_14d_data_port PORTA
#define somo_14d_data_pin 2

// don't define these if you don't want to use them

#define somo_14d_reset_port PORTA
#define somo_14d_reset_pin 3

#define somo_14d_busy_port PORTA
#define somo_14d_busy_pin 4

// - - - - - - - - - - - - - - - - - - - - 

*/

#ifndef __somo_14d_h
#define __somo_14d_h
#include "pic_utils.h"

#define SOMO_14D_PLAY_PAUSE_CMD 0xfffe
#define SOMO_14D_STOP_CMD		0xffff
#define SOMO_14D_VOLUME_CMD		0xfff0

void somo_14d_setup_io();
void somo_14d_reset();
void somo_14d_standby();
void somo_14d_wake();
void somo_14d_set_file_id(uns16 file_id);
void somo_14d_set_volume(uns8 level);
void somo_14d_play_pause();
void somo_14d_stop();
uns8 somo_14d_is_busy();

#ifndef somo_14d_clk_port
	#error "You haven't defined somo_14d_clk_port in your config.h!"
#endif

#ifndef somo_14d_data_port
	#error "You haven't defined somo_14d_data_port in your config.h!"
#endif

#ifndef somo_14d_clk_pin
	#error "You haven't defined somo_14d_clk_pin in your config.h!"
#endif

#ifndef somo_14d_data_pin
	#error "You haven't defined somo_14d_clk_pin in your config.h!"
#endif

#endif
