/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com

*/
/** \file tmp75.c
    \brief Routines to access TMP75 temperature sensor

*/
#include "tmp75.h"
#include "pic_serial.h"

uns8 tmp75_read(uns8 addr, uns8 pointer) {
uns8 data;

    i2c_start();
    i2c_send_byte(0x90 + addr);	//	w=0, write pointer
    
    i2c_send_byte(pointer);	// includes reading ack
    
	i2c_start();
    i2c_send_byte(0x90 + addr + 1);	// read
	
	data = i2c_receive_byte();


    i2c_stop();
    
    return data;
}

uns8 tmp75_write(uns8 addr, uns8 pointer, uns8 data) {

    i2c_start();
    i2c_send_byte(0x90 + addr);
    i2c_send_byte(pointer);

    i2c_send_byte(data);

    i2c_stop();
}


uns16 tmp75_read_16bit(uns8 addr, uns8 pointer) {
uns16 data;
    
    i2c_start();
    i2c_send_byte(0x90 + addr);	// LSB=0 == write
    i2c_send_byte(pointer);

	i2c_start();
    i2c_send_byte(0x90 + addr + 1);	// LSB=0 == read
	data = i2c_receive_byte();

	i2c_send_ack();
    
    data = data << 8 | i2c_receive_byte(); // reuse local variable

    i2c_send_ack();

    i2c_stop();
}

void tmp75_setup() {
	i2c_setup();
}

void  tmp75_set_config(uns8 addr, uns8 config)
{
	tmp75_write(addr, TMP75_CONFIG_REGISTER, config);
}

uns8 tmp75_get_config(uns8 addr)
{
	return tmp75_read(addr, TMP75_CONFIG_REGISTER);
}

void tmp75_convert_temp(uns8 addr) {
/// needs fixing
//	i2c_start();
//	i2c_send_byte(0x90 + addr); 
//	i2c_send_byte(ds1631_start_convert);	
//	i2c_stop();	
}

uns16 tmp75_get_temp(uns8 addr)
{
	return tmp75_read_16bit(addr, TMP75_TEMP_REGISTER);

}
