/*

Copyright (c) 2010, Embedded Adventures, www.embeddedadventures.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.

- Neither the name of Embedded Adventures nor the names of its contributors
  may be used to endorse or promote products derived from this software
  without specific prior written permission.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
THE POSSIBILITY OF SUCH DAMAGE.

Contact us at admin@embeddedadventures.com
*/

/** \file tmp75.h
    \brief Routines to access TMP75 temperature sensor

Put the following in your config.h
\code
// - - - - - - - - - - - - - - - - - - - - 
// TMP75 defines
// - - - - - - - - - - - - - - - - - - - - 

#define TMP75_ADDR 	0x00
\endcode

*/


#include "pic_utils.h"
#include "i2c.h"

#define TMP75_TEMP_REGISTER		0b00000000
#define TMP75_CONFIG_REGISTER	0b00000001
#define TMP75_TLOW_REGISTER		0b00000010
#define TMP75_THI_REGISTER		0b00000011

#define TMP75_CONF_OS	7
#define TMP75_CONF_R1	6
#define TMP75_CONF_R0	5
#define TMP75_CONF_F1	4
#define TMP75_CONF_F0	3
#define TMP75_CONF_POL	2
#define TMP75_CONF_TM	1
#define TMP75_CONF_SD	0




/**
	\brief Setup tmp75 ports and pins
	
*/
#define tmp75_setup() tmp75_setup_io()
void tmp75_setup_io(void);

/**
	\brief Set tmp75 config register
	
	Sets the tmp75 config register
	
*/
void tmp75_set_config(uns8 addr, uns8 config);

/**
	\brief Get tmp75 config register
	
	Gets the tmp75 config register (memory location 0x01)


*/
uns8 tmp75_get_config(uns8 addr);


/**
	\brief Read temperature from tmp75

	Returns 16bit raw temperature register from tmp75.

*/
uns16 tmp75_get_temp(uns8 addr);

/**
	\brief Start temperature conversion on tmp75
	
	This routine starts the temperature conversion in the tmp75. Issue
	this command before actually reading the temperature.
*/
void tmp75_convert_temp(uns8 addr);
